@@ -2,13 +2,19 @@ mod frame_buffer;
 mod io;
 
 pub use frame_buffer::{FrameBuffer, FrameBufferSource};
-pub use io::{LoadableAsset, SeekFrom};
+pub use io::{DataRecorder, LoadableAsset, SeekFrom, SeekableAsset};
+
+pub use io::BufferCursor;
 
 pub enum Snapshot<LoadableAssetImpl: LoadableAsset> {
     Sna(LoadableAssetImpl),
     // TODO(#55): Implement SLT snapshot format support
 }
 
+pub enum SnapshotRecorder<DataRecorderImpl: DataRecorder> {
+    Sna(DataRecorderImpl),
+}
+
 pub enum Tape<LoadableAssetImpl: LoadableAsset> {
     Tap(LoadableAssetImpl),
     // TODO(#56): Implement TZX tape format support
@@ -36,9 +42,7 @@ pub trait Host {
     /// context objects for host-defined emulator parts construction (e.g. FrameBuffers)
     type Context: HostContext<Self>;
     /// File-like type implementation for tape loading
-    type TapeAsset: LoadableAsset;
-    /// File-like type implementation for snapshot loading
-    type SnapshotAsset: LoadableAsset;
+    type TapeAsset: LoadableAsset + SeekableAsset;
     /// File-like type implementation for rom loading
     type RomSet: RomSet;
     /// Frame buffer implementation
