@@ -8,10 +8,14 @@ use crate::{
     zx::{
         constants::{ADDR_LD_BREAK, CANVAS_HEIGHT, CLOCKS_PER_COL},
         events::EmulationEvents,
-        joy::kempston::KempstonJoy,
-        keys::ZXKey,
+        joy::{
+            kempston::KempstonJoy,
+            sinclair::{self, SinclairJoyNum, SinclairKey},
+        },
+        keys::{CompoundKey, ZXKey},
         machine::ZXMachine,
         memory::{Page, RamType, RomType, ZXMemory, PAGE_SIZE},
+        mouse::kempston::{KempstonMouse, KempstonMouseButton, KempstonMouseWheelDirection},
         tape::{TapeImpl, ZXTape},
         video::{colors::ZXColor, screen::ZXScreen},
     },
@@ -34,9 +38,13 @@ pub(crate) struct ZXController<H: Host> {
     #[cfg(feature = "precise-border")]
     pub border: ZXBorder<H::FrameBuffer>,
     pub kempston: Option<KempstonJoy>,
+    pub mouse: Option<KempstonMouse>,
     #[cfg(feature = "sound")]
     pub mixer: ZXMixer,
     pub keyboard: [u8; 8],
+    pub keyboard_extended: [u8; 8],
+    pub keyboard_sinclair: [u8; 8],
+    pub caps_shift_modifier_mask: u32,
     // current border color
     pub border_color: ZXColor,
     // clocls count from frame start
@@ -50,6 +58,7 @@ pub(crate) struct ZXController<H: Host> {
     ear: bool,
     paging_enabled: bool,
     screen_bank: u8,
+    current_port_7ffd: u8,
     // Z80 module expected controller implementation without errors,
     // so we need to store the internal errors manually. For sake of simplicity,
     // Only last error is saved
@@ -72,12 +81,19 @@ impl<H: Host> ZXController<H> {
                 screen_bank = 5;
             }
         };
-        let kempston = if settings.enable_kempston {
+
+        let kempston = if settings.kempston_enabled {
             Some(KempstonJoy::default())
         } else {
             None
         };
 
+        let mouse = if settings.mouse_enabled {
+            Some(KempstonMouse::default())
+        } else {
+            None
+        };
+
         let screen = ZXScreen::new(settings.machine, host_context.frame_buffer_context());
         #[cfg(feature = "precise-border")]
         let border = ZXBorder::new(settings.machine, host_context.frame_buffer_context());
@@ -92,9 +108,13 @@ impl<H: Host> ZXController<H> {
             #[cfg(feature = "precise-border")]
             border,
             kempston,
+            mouse,
             #[cfg(feature = "sound")]
             mixer,
             keyboard: [0xFF; 8],
+            keyboard_extended: [0xFF; 8],
+            keyboard_sinclair: [0xFF; 8],
+            caps_shift_modifier_mask: 0,
             border_color: ZXColor::Black,
             frame_clocks: Clocks(0),
             passed_frames: 0,
@@ -104,6 +124,7 @@ impl<H: Host> ZXController<H> {
             ear: false,
             paging_enabled: paging,
             screen_bank,
+            current_port_7ffd: 0,
             last_emulation_error: None,
         };
 
@@ -160,10 +181,59 @@ impl<H: Host> ZXController<H> {
 
     /// Changes key state in controller
     pub fn send_key(&mut self, key: ZXKey, pressed: bool) {
-        let row_id = key.row_id();
-        self.keyboard[row_id] &= !key.mask();
-        if !pressed {
-            self.keyboard[row_id] |= key.mask();
+        if pressed {
+            self.keyboard[key.row_id()] &= !key.mask();
+            return;
+        }
+        self.keyboard[key.row_id()] |= key.mask();
+    }
+
+    pub fn send_sinclair_key(&mut self, num: SinclairJoyNum, key: SinclairKey, pressed: bool) {
+        let key = sinclair::sinclair_event_to_zx_key(key, num);
+        if pressed {
+            self.keyboard_sinclair[key.row_id()] &= !key.mask();
+            return;
+        }
+        self.keyboard_sinclair[key.row_id()] |= key.mask();
+    }
+
+    pub fn send_compound_key(&mut self, key: CompoundKey, pressed: bool) {
+        let mut dummy_modifier_mask = 0;
+        let modifier_mask = match key.modifier_key() {
+            ZXKey::Shift => &mut self.caps_shift_modifier_mask,
+            _ => &mut dummy_modifier_mask,
+        };
+        let primary_key = key.primary_key();
+        let modifier_key = key.modifier_key();
+
+        if pressed {
+            *modifier_mask |= key.modifier_mask();
+            self.keyboard_extended[primary_key.row_id()] &= !primary_key.mask();
+            self.keyboard_extended[modifier_key.row_id()] &= !modifier_key.mask();
+        } else {
+            *modifier_mask &= !key.modifier_mask();
+            if *modifier_mask == 0 {
+                self.keyboard_extended[modifier_key.row_id()] |= modifier_key.mask();
+            }
+            self.keyboard_extended[primary_key.row_id()] |= primary_key.mask();
+        }
+    }
+
+    pub fn send_mouse_button(&mut self, button: KempstonMouseButton, pressed: bool) {
+        if let Some(mouse) = &mut self.mouse {
+            mouse.send_button(button, pressed);
+        }
+    }
+
+    pub fn send_mouse_wheel(&mut self, dir: KempstonMouseWheelDirection) {
+        if let Some(mouse) = &mut self.mouse {
+            mouse.send_wheel(dir);
+        }
+    }
+
+    pub fn send_mouse_pos_diff(&mut self, x: i8, y: i8) {
+        if let Some(mouse) = &mut self.mouse {
+            mouse.send_pos_diff(x, y);
         }
     }
 
@@ -268,10 +338,11 @@ impl<H: Host> ZXController<H> {
         self.frame_clocks
     }
 
-    fn write_7ffd(&mut self, val: u8) {
+    pub fn write_7ffd(&mut self, val: u8) {
         if !self.paging_enabled {
             return;
         }
+        self.current_port_7ffd = val;
         // remap top 16K of the ram
         self.memory.remap(3, Page::Ram(val & 0x07));
         // third block is not pageable
@@ -287,6 +358,10 @@ impl<H: Host> ZXController<H> {
         }
     }
 
+    pub fn read_7ffd(&self) -> u8 {
+        self.current_port_7ffd
+    }
+
     #[cfg(all(feature = "sound", feature = "ay"))]
     fn read_ay_port(&mut self) -> u8 {
         self.mixer.ay.read()
@@ -326,6 +401,24 @@ impl<H: Host> ZXController<H> {
     pub(crate) fn take_last_emulation_error(&mut self) -> Option<Error> {
         self.last_emulation_error.take()
     }
+
+    pub(crate) fn refresh_memory_dependent_devices(&mut self) {
+        match self.machine {
+            ZXMachine::Sinclair48K => {
+                for (idx, data) in self.memory.ram_page_data(0).iter().enumerate() {
+                    self.screen.update(idx as u16, 0, *data);
+                }
+            }
+            ZXMachine::Sinclair128K => {
+                for (idx, data) in self.memory.ram_page_data(5).iter().enumerate() {
+                    self.screen.update(idx as u16, 5, *data);
+                }
+                for (idx, data) in self.memory.ram_page_data(7).iter().enumerate() {
+                    self.screen.update(idx as u16, 7, *data);
+                }
+            }
+        }
+    }
 }
 
 impl<H: Host> Z80Bus for ZXController<H> {
@@ -416,7 +509,9 @@ impl<H: Host> Z80Bus for ZXController<H> {
             for n in 0..8 {
                 // if bit of row reset
                 if ((h >> n) & 0x01) == 0 {
-                    tmp &= self.keyboard[n];
+                    let keyboard_byte =
+                        self.keyboard[n] & self.keyboard_extended[n] & self.keyboard_sinclair[n];
+                    tmp &= keyboard_byte;
                 }
             }
             // invert bit 6 if mic_hw active;
@@ -425,14 +520,16 @@ impl<H: Host> Z80Bus for ZXController<H> {
             }
             // 5 and 7 unused
             tmp
+        } else if self.mouse.is_some() && (port & 0x0121 == 0x0001) {
+            self.mouse.as_ref().unwrap().buttons_port
+        } else if self.mouse.is_some() && (port & 0x0521 == 0x0101) {
+            self.mouse.as_ref().unwrap().x_pos_port
+        } else if self.mouse.is_some() && (port & 0x0521 == 0x0501) {
+            self.mouse.as_ref().unwrap().y_pos_port
         } else if port & 0xC002 == 0xC000 {
             self.read_ay_port()
-        } else if self.kempston.is_some() && (port & 0x0020 == 0) {
-            if let Some(ref joy) = self.kempston {
-                joy.read()
-            } else {
-                unreachable!()
-            }
+        } else if self.kempston.is_some() && (port & 0x00E0 == 0) {
+            self.kempston.as_ref().unwrap().read()
         } else {
             self.floating_bus_value()
         };
